NAME    Chess       
DESCRIPTION 'GNU Chess 3.2 for MS WINDOWS -- Daryl Baker'
EXETYPE WINDOWS
STUB    'WINSTUB.EXE' 
CODE    MOVEABLE discardable
DATA    PRELOAD MOVEABLE MULTIPLE
SEGMENTS
        _TEXT           PRELOAD MOVEABLE DISCARDABLE

HEAPSIZE  1024
STACKSIZE 32768
EXPORTS
    ChessWndProc          @1
    About                 @2
    ColorDlgProc          @4
    TimeControlDlgProc    @5
    FileOpenDlgProc       @6
    FileSaveDlgProc       @7
    NumberDlgDlgProc      @8
    ReviewDlgProc         @9
    StatDlgProc           @10
    TestDlgProc           @11
    PromoteDlgProc        @12

