/*
  C source for GNU CHESS

  Revision: 1990-09-30
  Modified by Daryl Baker for use in MS WINDOWS environment

  This file is part of CHESS.

  CHESS is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY.  No author or distributor accepts responsibility to anyone for
  the consequences of using it or for whether it serves any particular
  purpose or works at all, unless he says so in writing.  Refer to the CHESS
  General Public License for full details.

  Everyone is granted permission to copy, modify and redistribute CHESS, but
  only under the conditions described in the CHESS General Public License.
  A copy of this license is supposed to have been given to you along with
  CHESS so you can know your rights and responsibilities.  It should be in a
  file named COPYING.  Among other things, the copyright notice and this
  notice must be preserved on all copies.
*/

#define IDD_HEADING  200
#define IDD_OK       201
#define IDD_CANCEL   202

#define CNT_BLACK    100
#define CNT_BLUE     101
#define CNT_GREEN    102
#define CNT_CYAN     103
#define CNT_RED      104
#define CNT_PINK     105
#define CNT_YELLOW   106
#define CNT_PALEGRAY 107
#define CNT_DARKGRAY 108
#define CNT_DARKBLUE 109
#define CNT_DARKGREEN   110
#define CNT_DARKCYAN    111
#define CNT_DARKRED     112
#define CNT_DARKPINK    113
#define CNT_BROWN       114
#define CNT_WHITE       115

